'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _cspaceRefname = require('cspace-refname');

var _ReadOnlyInput = require('./ReadOnlyInput');

var _ReadOnlyInput2 = _interopRequireDefault(_ReadOnlyInput);

var _FilteringDropdownMenuInput = require('./FilteringDropdownMenuInput');

var _FilteringDropdownMenuInput2 = _interopRequireDefault(_FilteringDropdownMenuInput);

var _QuickAdd = require('./QuickAdd');

var _QuickAdd2 = _interopRequireDefault(_QuickAdd);

var _parseResourceID = require('../helpers/parseResourceID');

var _parseResourceID2 = _interopRequireDefault(_parseResourceID);

var _pathHelpers = require('../helpers/pathHelpers');

var _AutocompleteInput = require('../../styles/cspace-input/AutocompleteInput.css');

var _AutocompleteInput2 = _interopRequireDefault(_AutocompleteInput);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var propTypes = _extends({}, _FilteringDropdownMenuInput2.default.propTypes, {
  addTerm: _propTypes2.default.func,
  findMatchingTerms: _propTypes2.default.func,
  formatAddPrompt: _propTypes2.default.func,
  formatMoreCharsRequiredMessage: _propTypes2.default.func,
  formatSearchResultMessage: _propTypes2.default.func,
  formatSourceName: _propTypes2.default.func,
  matches: _propTypes2.default.object,
  minLength: _propTypes2.default.number,
  recordTypes: _propTypes2.default.object,
  showQuickAdd: _propTypes2.default.bool,
  readOnly: _propTypes2.default.bool,
  source: _propTypes2.default.string,
  matchFilter: _propTypes2.default.func
});

var defaultProps = {
  formatMoreCharsRequiredMessage: function formatMoreCharsRequiredMessage() {
    return 'Continue typing to find matching terms';
  },
  formatSearchResultMessage: function formatSearchResultMessage(count) {
    var matches = count === 1 ? 'matching term' : 'matching terms';
    var num = count === 0 ? 'No' : count;

    return num + ' ' + matches + ' found';
  },
  matchFilter: function matchFilter() {
    return true;
  },
  // TODO: Make configurable
  minLength: 3,
  showQuickAdd: true
};

var getOptions = function getOptions(sourceID, matches, partialTerm, matchFilter) {
  var sources = (0, _parseResourceID2.default)(sourceID);
  var options = [];

  if (matches) {
    var partialTermMatch = matches.get(partialTerm);

    if (partialTermMatch) {
      sources.forEach(function (source) {
        var recordType = source.recordType,
            vocabulary = source.vocabulary;


        var sourceMatch = partialTermMatch.getIn([recordType, vocabulary]);

        if (sourceMatch) {
          var items = sourceMatch.get('items');

          if (items) {
            var sourceOptions = items.filter(matchFilter).map(function (item) {
              return {
                value: item.refName,
                label: (0, _cspaceRefname.getDisplayName)(item.refName),
                meta: item
              };
            });

            options.push.apply(options, _toConsumableArray(sourceOptions));
          }
        }
      });
    }
  }

  return options;
};

var isPending = function isPending(sourceID, matches, partialTerm) {
  var sources = (0, _parseResourceID2.default)(sourceID);
  var foundPending = false;

  if (matches) {
    var partialTermMatch = matches.get(partialTerm);

    if (partialTermMatch) {
      sources.forEach(function (source) {
        var recordType = source.recordType,
            vocabulary = source.vocabulary;


        var sourceMatch = partialTermMatch.getIn([recordType, vocabulary]);

        if (sourceMatch) {
          foundPending = foundPending || sourceMatch.get('isSearchPending') || sourceMatch.get('isAddPending');
        }
      });
    }
  }

  return foundPending;
};

var getNewTerm = function getNewTerm(sourceID, matches, partialTerm) {
  var sources = (0, _parseResourceID2.default)(sourceID);

  var newTerm = null;

  if (matches) {
    var partialTermMatch = matches.get(partialTerm);

    if (partialTermMatch) {
      sources.forEach(function (source) {
        var recordType = source.recordType,
            vocabulary = source.vocabulary;


        var sourceMatch = partialTermMatch.getIn([recordType, vocabulary]);

        if (sourceMatch) {
          newTerm = newTerm || sourceMatch.get('newTerm');
        }
      });
    }
  }

  return newTerm;
};

var AutocompleteInput = function (_Component) {
  _inherits(AutocompleteInput, _Component);

  function AutocompleteInput(props) {
    _classCallCheck(this, AutocompleteInput);

    var _this = _possibleConstructorReturn(this, (AutocompleteInput.__proto__ || Object.getPrototypeOf(AutocompleteInput)).call(this, props));

    _this.findMatchingTerms = _this.findMatchingTerms.bind(_this);
    _this.handleDropdownInputCommit = _this.handleDropdownInputCommit.bind(_this);
    _this.handleFilteringDropdownMenuInputRef = _this.handleFilteringDropdownMenuInputRef.bind(_this);
    _this.handleQuickAddBeforeItemFocusChange = _this.handleQuickAddBeforeItemFocusChange.bind(_this);
    _this.handleDropdownBeforeItemFocusChange = _this.handleDropdownBeforeItemFocusChange.bind(_this);
    _this.handleQuickAddRef = _this.handleQuickAddRef.bind(_this);
    _this.handleDropdownMenuInputRef = _this.handleDropdownMenuInputRef.bind(_this);
    _this.handleFocusPopup = _this.handleFocusPopup.bind(_this);

    _this.state = {
      partialTerm: null,
      value: props.value
    };
    return _this;
  }

  _createClass(AutocompleteInput, [{
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var newTerm = getNewTerm(nextProps.source, nextProps.matches, this.state.partialTerm);

      var hadNewTerm = getNewTerm(this.props.source, this.props.matches, this.state.partialTerm);

      if (newTerm && !hadNewTerm) {
        var refName = newTerm.getIn(['document', 'ns2:collectionspace_core', 'refName']);
        var uri = newTerm.getIn(['document', 'ns2:collectionspace_core', 'uri']);
        var csid = uri.substring(uri.lastIndexOf('/') + 1);

        this.commit(refName, { csid: csid });
        this.filteringDropdownMenuInput.close();
      } else {
        var newState = {
          value: nextProps.value
        };

        if (!isPending(nextProps.source, nextProps.matches, this.state.partialTerm)) {
          newState.options = getOptions(nextProps.source, nextProps.matches, this.state.partialTerm, nextProps.matchFilter);
        }

        this.setState(newState);
      }
    }
  }, {
    key: 'commit',
    value: function commit(value, meta) {
      this.setState({
        options: [],
        partialTerm: null,
        value: value
      });

      var onCommit = this.props.onCommit;


      var csid = meta ? meta.csid : undefined;

      if (onCommit) {
        onCommit((0, _pathHelpers.getPath)(this.props), value, csid);
      }
    }
  }, {
    key: 'findMatchingTerms',
    value: function findMatchingTerms(partialTerm) {
      var _props = this.props,
          source = _props.source,
          findMatchingTerms = _props.findMatchingTerms,
          matches = _props.matches,
          minLength = _props.minLength,
          matchFilter = _props.matchFilter;


      var newState = {
        partialTerm: partialTerm
      };

      var searchNeeded = findMatchingTerms && partialTerm && partialTerm.length >= minLength && (!matches || !matches.has(partialTerm));

      if (searchNeeded) {
        // TODO: Pause to debounce
        findMatchingTerms(partialTerm);
      } else {
        newState.options = getOptions(source, matches, partialTerm, matchFilter);
      }

      this.setState(newState);
    }
  }, {
    key: 'handleDropdownInputCommit',
    value: function handleDropdownInputCommit(path, value, meta) {
      this.commit(value, meta);
    }
  }, {
    key: 'handleFilteringDropdownMenuInputRef',
    value: function handleFilteringDropdownMenuInputRef(ref) {
      this.filteringDropdownMenuInput = ref;
    }
  }, {
    key: 'handleDropdownMenuInputRef',
    value: function handleDropdownMenuInputRef(ref) {
      this.dropdownMenuInput = ref;
    }
  }, {
    key: 'handleQuickAddRef',
    value: function handleQuickAddRef(ref) {
      this.quickAdd = ref;
    }
  }, {
    key: 'handleQuickAddBeforeItemFocusChange',
    value: function handleQuickAddBeforeItemFocusChange(currentFocusedIndex, nextFocusedIndex, eventKey) {
      var options = this.state.options;


      if (this.dropdownMenuInput && options && options.length > 0) {
        if (nextFocusedIndex === 0 && eventKey === 'ArrowDown') {
          this.dropdownMenuInput.focusMenu(0);
          return null;
        } else if (currentFocusedIndex <= 0 && eventKey === 'ArrowUp') {
          this.dropdownMenuInput.focusMenu(-1);
          return null;
        }
      }

      return nextFocusedIndex;
    }
  }, {
    key: 'handleDropdownBeforeItemFocusChange',
    value: function handleDropdownBeforeItemFocusChange(currentFocusedIndex, nextFocusedIndex, eventKey) {
      if (this.quickAdd) {
        if (nextFocusedIndex === 0 && eventKey === 'ArrowDown') {
          this.quickAdd.focusMenu(0);
          return null;
        } else if (currentFocusedIndex <= 0 && eventKey === 'ArrowUp') {
          this.quickAdd.focusMenu(-1);
          return null;
        }
      }

      return nextFocusedIndex;
    }
  }, {
    key: 'handleFocusPopup',
    value: function handleFocusPopup() {
      var options = this.state.options;


      if (options && options.length > 0) {
        if (this.dropdownMenuInput) {
          this.dropdownMenuInput.focusMenu(0);
        }
      } else if (this.quickAdd) {
        this.quickAdd.focusMenu(0);
      }
    }
  }, {
    key: 'renderQuickAdd',
    value: function renderQuickAdd() {
      var _props2 = this.props,
          addTerm = _props2.addTerm,
          formatAddPrompt = _props2.formatAddPrompt,
          formatSourceName = _props2.formatSourceName,
          minLength = _props2.minLength,
          recordTypes = _props2.recordTypes,
          showQuickAdd = _props2.showQuickAdd,
          source = _props2.source;
      var partialTerm = this.state.partialTerm;


      if (showQuickAdd && partialTerm && partialTerm.length >= minLength) {
        return _react2.default.createElement(_QuickAdd2.default, {
          add: addTerm,
          displayName: partialTerm,
          formatAddPrompt: formatAddPrompt,
          formatDestinationName: formatSourceName,
          recordTypes: recordTypes,
          to: source,
          onBeforeItemFocusChange: this.handleQuickAddBeforeItemFocusChange,
          ref: this.handleQuickAddRef
        });
      }

      return null;
    }
  }, {
    key: 'renderReadOnly',
    value: function renderReadOnly() {
      var value = this.state.value;


      return _react2.default.createElement(_ReadOnlyInput2.default, {
        value: (0, _cspaceRefname.getDisplayName)(value)
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _props3 = this.props,
          formatMoreCharsRequiredMessage = _props3.formatMoreCharsRequiredMessage,
          formatSearchResultMessage = _props3.formatSearchResultMessage,
          matches = _props3.matches,
          minLength = _props3.minLength,
          readOnly = _props3.readOnly,
          source = _props3.source,
          addTerm = _props3.addTerm,
          findMatchingTerms = _props3.findMatchingTerms,
          formatAddPrompt = _props3.formatAddPrompt,
          formatSourceName = _props3.formatSourceName,
          recordTypes = _props3.recordTypes,
          showQuickAdd = _props3.showQuickAdd,
          matchFilter = _props3.matchFilter,
          remainingProps = _objectWithoutProperties(_props3, ['formatMoreCharsRequiredMessage', 'formatSearchResultMessage', 'matches', 'minLength', 'readOnly', 'source', 'addTerm', 'findMatchingTerms', 'formatAddPrompt', 'formatSourceName', 'recordTypes', 'showQuickAdd', 'matchFilter']);

      var _state = this.state,
          options = _state.options,
          partialTerm = _state.partialTerm,
          value = _state.value;


      if (readOnly) {
        return this.renderReadOnly();
      }

      var moreCharsRequired = typeof partialTerm !== 'undefined' && partialTerm !== null && partialTerm.length < minLength;

      var formatStatusMessage = moreCharsRequired ? formatMoreCharsRequiredMessage : formatSearchResultMessage;

      var className = isPending(source, matches, partialTerm) ? _AutocompleteInput2.default.searching : _AutocompleteInput2.default.normal;

      return _react2.default.createElement(_FilteringDropdownMenuInput2.default, _extends({}, remainingProps, {
        className: className,
        filter: this.findMatchingTerms,
        formatStatusMessage: formatStatusMessage,
        menuFooter: this.renderQuickAdd(),
        options: options,
        ref: this.handleFilteringDropdownMenuInputRef,
        value: value,
        valueLabel: (0, _cspaceRefname.getDisplayName)(value),
        onCommit: this.handleDropdownInputCommit,
        onBeforeItemFocusChange: this.handleDropdownBeforeItemFocusChange,
        onMount: this.handleDropdownMenuInputRef,
        focusPopup: this.handleFocusPopup
      }));
    }
  }]);

  return AutocompleteInput;
}(_react.Component);

exports.default = AutocompleteInput;


AutocompleteInput.propTypes = propTypes;
AutocompleteInput.defaultProps = defaultProps;